require 'spec_helper'
require 'htauth/passwd_file'
require 'tempfile'

describe HTAuth::PasswdFile do

  before(:each) do
    @tf             = Tempfile.new("rpasswrd-passwd")
    @tf.write(IO.read(PASSWD_ORIGINAL_TEST_FILE))
    @tf.close
    @passwd_file    = HTAuth::PasswdFile.new(@tf.path)

    @tf2                = Tempfile.new("rpasswrd-passwd-empty")
    @tf2.close
    @empty_passwd_file  = HTAuth::PasswdFile.new(@tf2.path)
  end

  after(:each) do
    @tf2.close(true)
    @tf.close(true)
  end

  it "can add a new entry to an already existing passwd file" do
    @passwd_file.add_or_update("charlie", "c secret", "sha1")
    _(@passwd_file.contents).must_equal IO.read(PASSWD_ADD_TEST_FILE)
  end

  it "can tell if an entry already exists in the passwd file" do
    _(@passwd_file.has_entry?("alice")).must_equal true
    _(@passwd_file.has_entry?("david")).must_equal false
  end

  it "can update an entry in an already existing passwd file, algorithm can change" do
    @passwd_file.add_or_update("alice", "a new secret", "sha1")
    _(@passwd_file.contents).must_equal IO.read(PASSWD_UPDATE_TEST_FILE)
  end

  it "can update an entry in an already existing passwd file, algorithm and arguments can change" do
    @passwd_file.add_or_update("brenda", "b secret", "bcrypt")
    entry = @passwd_file.fetch("brenda")
    _(entry.algorithm.cost).must_equal(::HTAuth::Bcrypt::DEFAULT_APACHE_COST)
    @passwd_file.add_or_update("brenda", "b secret", "bcrypt", :cost => 12)
    entry = @passwd_file.fetch("brenda")
    _(entry.algorithm.cost).must_equal(12)
  end

  it "fetches a copy of an entry" do
    _(@passwd_file.fetch("alice").to_s).must_equal "alice:$apr1$DghnA...$CsPcgerfsI/Ryy0AOAJtb0"
  end

  it "raises an error if an attempt is made to alter a non-existent file" do
    _ { HTAuth::PasswdFile.new("some-file") }.must_raise(HTAuth::FileAccessError)
  end

  # this test will only work on systems that have /etc/ssh_host_rsa_key 
  it "raises an error if an attempt is made to open a file where no permissions are granted" do
    _ { HTAuth::PasswdFile.new("/etc/ssh_host_rsa_key") }.must_raise(HTAuth::FileAccessError)
  end

  it "deletes an entry" do
    @passwd_file.delete("bob")
    _(@passwd_file.contents).must_equal IO.read(PASSWD_DELETE_TEST_FILE)
  end

  it "checks authentication of an entry - true" do
    _(@passwd_file.authenticated?("alice", "a secret")).must_equal true
  end

  it "checks authentication of an entry - false" do
    _(@passwd_file.authenticated?("alice", "the wrong secret")).must_equal false
  end


  it "is usable in a ruby manner and yields itself when opened" do
    HTAuth::PasswdFile.open(@tf.path) do |pf|
      pf.add_or_update("alice", "a new secret", "md5")
      pf.delete('bob')
    end
    lines = IO.readlines(@tf.path)
    _(lines.size).must_equal 1
    _(lines.first.split(':').first).must_equal "alice"
    _(lines.first.split(':').last).must_match( /\$apr1\$/ )
  end
end
